@@ -2,13 +2,16 @@ from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.search import SearchVector
 from django.db.models import Prefetch, Q
 from django.utils.timezone import now
+from django.utils.translation import ugettext_lazy as _
 from django_filters import rest_framework as filters
 from rest_framework.exceptions import ValidationError
 
+from config import settings as store_settings
 from ob_dj_store.core.stores.models import (
     Category,
     Favorite,
     Order,
+    PaymentMethod,
     Product,
     ProductVariant,
     Store,
@@ -73,6 +76,10 @@ class ProductFilter(filters.FilterSet):
     """Product filters"""
 
     category = filters.CharFilter(method="by_category")
+    store = filters.CharFilter(
+        method="by_store",
+        help_text=_("store id param to get only the variants in this store"),
+    )
 
     class Meta:
         model = Product
@@ -85,6 +92,19 @@ class ProductFilter(filters.FilterSet):
     def by_category(self, queryset, name, value):
         return queryset.filter(category__name__iexact=value)
 
+    def by_store(self, queryset, name, value):
+        queryset = queryset.filter(
+            product_variants__inventories__store=value
+        ).prefetch_related(
+            Prefetch(
+                "product_variants",
+                queryset=ProductVariant.objects.filter(
+                    inventories__store=value
+                ).distinct(),
+            )
+        )
+        return queryset
+
 
 class VariantFilter(filters.FilterSet):
     """Variant filters"""
@@ -128,6 +148,7 @@ class CategoryFilter(filters.FilterSet):
                         is_active=True,
                     ).distinct(),
                 ),
+                "subcategories__products__images",
             )
             .distinct()
         )
@@ -174,3 +195,25 @@ class FavoriteFilter(filters.FilterSet):
             content_type=content_type,
             object_id__in=products_ids,
         )
+
+
+class PaymentMethodFilter(filters.FilterSet):
+    store = filters.CharFilter(method="by_store")
+    is_digital = filters.BooleanFilter(method="by_digital")
+
+    class Meta:
+        models = PaymentMethod
+
+    def by_store(self, queryset, name, value):
+        return queryset.filter(stores=value)
+
+    def by_digital(self, queryset, name, value):
+        if value:
+            return queryset.filter(
+                payment_provider__in=[
+                    store_settings.TAP_CREDIT_CARD,
+                    store_settings.TAP_KNET,
+                    store_settings.TAP_ALL,
+                ]
+            )
+        return queryset
