@@ -1,5 +1,4 @@
 import calendar
-import sys
 import typing
 from datetime import time, timedelta
 
@@ -7,7 +6,9 @@ from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.gis.geos import Point
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
+from django.db import models
 from django.shortcuts import get_object_or_404
+from django.utils.module_loading import import_string
 from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
 from rest_framework import serializers
@@ -39,6 +40,9 @@ from ob_dj_store.core.stores.models import (
     ShippingMethod,
     Store,
     Tax,
+    Wallet,
+    WalletMedia,
+    WalletTransaction,
 )
 from ob_dj_store.core.stores.models._inventory import Inventory
 from ob_dj_store.core.stores.utils import distance
@@ -51,6 +55,7 @@ class AttributeChoiceSerializer(serializers.ModelSerializer):
             "id",
             "name",
             "price",
+            "label",
             "is_default",
         )
 
@@ -147,6 +152,7 @@ class OrderSerializer(serializers.ModelSerializer):
     items = OrderItemSerializer(many=True, read_only=True)
     history = OrderHistorySerializer(many=True, read_only=True)
     estimated_timeline = serializers.SerializerMethodField()
+    store_name = serializers.SerializerMethodField()
 
     class Meta:
         model = Order
@@ -169,6 +175,7 @@ class OrderSerializer(serializers.ModelSerializer):
             "extra_infos",
             "created_at",
             "updated_at",
+            "store_name",
         )
         extra_kwargs = {
             "customer": {"read_only": True},
@@ -199,9 +206,11 @@ class OrderSerializer(serializers.ModelSerializer):
             timeline["ready"] = timeline["delivering"] or timeline["order_ready"]
         return timeline
 
+    def get_store_name(self, obj):
+        return obj.store.name if obj.store else None
+
     def to_representation(self, instance):
         data = super().to_representation(instance)
-        data["store_name"] = instance.store.name
         return data
 
     def _get_store(self):
@@ -215,30 +224,29 @@ class OrderSerializer(serializers.ModelSerializer):
     def validate(self, attrs):
         user = self.context["request"].user
         attrs["store"] = self._get_store()
+        if not "extra_infos" in attrs:
+            attrs["extra_infos"] = {}
         stores = Store.objects.filter(store_items__cart=user.cart).distinct()
         if not store_settings.DIFFERENT_STORE_ORDERING and len(stores) > 1:
             raise ValidationError(_("You cannot order from different stores"))
-        if "extra_infos" in attrs:
+        gift_details = attrs["extra_infos"].get("gift_details", None)
+        if gift_details:
             errors = []
-            data = attrs["extra_infos"]
             for key in settings.DIGITAL_PRODUCTS_REQUIRED_KEYS:
-                if not (key in data.keys() and len(str(data.get(key)))):
+                if not (key in gift_details.keys() and len(str(gift_details.get(key)))):
                     errors.append({key: "This field should be filled."})
             if len(errors) > 0:
                 raise serializers.ValidationError(errors)
             try:
-                variant = ProductVariant.objects.get(
-                    id=attrs["extra_infos"]["digital_product"]
-                )
+                variant = ProductVariant.objects.get(id=gift_details["digital_product"])
             except ObjectDoesNotExist:
                 raise serializers.ValidationError(_("Digital product does not exist!"))
             if variant.product.type == Product.ProductTypes.PHYSICAL:
                 raise serializers.ValidationError(
                     _("You must not fill extra infos for physical products.")
                 )
-
         # The Cart items must not be empty
-        elif not "extra_infos" in attrs and not user.cart.items.exists():
+        elif not user.cart.items.exists():
             raise serializers.ValidationError(_("The Cart must not be empty"))
 
         if "pickup_time" in attrs:
@@ -288,7 +296,7 @@ class OrderSerializer(serializers.ModelSerializer):
         if payment_method:
             if payment_method.payment_provider == store_settings.WALLET:
                 try:
-                    wallet = user.wallets.get(country=attrs["store"].address.country)
+                    wallet = user.wallets.get(currency=attrs["store"].currency)
                 except ObjectDoesNotExist:
                     raise serializers.ValidationError(
                         {
@@ -297,18 +305,26 @@ class OrderSerializer(serializers.ModelSerializer):
                             )
                         }
                     )
-                if "extra_infos" in attrs:
-                    amount = attrs["extra_infos"]["price"]
+
+                if gift_details:
+                    amount = gift_details["price"]
                 else:
                     amount = user.cart.total_price
                 if wallet.balance < amount:
                     raise serializers.ValidationError(
                         {"wallet": _("Insufficient Funds")},
                     )
+            elif payment_method.payment_provider == store_settings.GIFT:
+                if not attrs["extra_infos"].get("gift_card", None):
+                    raise serializers.ValidationError(
+                        {"gift": _("The Gift card is missing")}
+                    )
 
         return super().validate(attrs)
 
     def perform_payment(self, amount, payment_method, order_store, orders):
+        from ob_dj_store.core.stores.gateway.tap.utils import TapException
+
         try:
             payment = Payment.objects.create(
                 user=self.context["request"].user,
@@ -319,6 +335,8 @@ class OrderSerializer(serializers.ModelSerializer):
             )
         except ValidationError as err:
             raise serializers.ValidationError(_(str(err)))
+        except TapException as err:
+            raise serializers.ValidationError({"tap": _(str(err))})
         return {
             "orders": orders,
             "payment_url": payment.payment_url,
@@ -328,11 +346,10 @@ class OrderSerializer(serializers.ModelSerializer):
         user = self.context["request"].user
         orders = []
         order_store = validated_data["store"]
-        if "extra_infos" in validated_data:
-            amount = validated_data["extra_infos"]["price"]
-            variant = ProductVariant.objects.get(
-                id=validated_data["extra_infos"]["digital_product"]
-            )
+        gift_details = validated_data["extra_infos"].get("gift_details", None)
+        if gift_details:
+            amount = gift_details["price"]
+            variant = ProductVariant.objects.get(id=gift_details["digital_product"])
             order = Order.objects.create(customer=user, **validated_data)
             order_item = OrderItem.objects.create(
                 order=order,
@@ -371,7 +388,14 @@ class OrderSerializer(serializers.ModelSerializer):
 class CreateOrderResponseSerializer(serializers.Serializer):
     orders = OrderSerializer(many=True, read_only=True)
     payment_url = serializers.CharField(read_only=True)
-    extra_infos = serializers.JSONField(required=False)
+    extra_infos = serializers.JSONField(
+        required=False,
+        help_text=f"""
+                gift_details :  {",".join(settings.DIGITAL_PRODUCTS_REQUIRED_KEYS)}  \n
+                gift_card : the id of the gift_card
+
+                    """,
+    )
 
     # write only fields
     shipping_method = serializers.IntegerField(write_only=True, required=False)
@@ -402,6 +426,7 @@ class ProductAttributeSerializer(serializers.ModelSerializer):
             "is_mandatory",
             "attribute_choices",
             "type",
+            "label",
             "min",
             "max",
         )
@@ -421,7 +446,10 @@ class ProductVariantSerializer(serializers.ModelSerializer):
             "product_attributes",
             "is_primary",
             "active_inventories",
+            "image",
+            "image_thumbnail_medium",
         )
+        extra_kwargs = {"image_thumbnail_medium": {"read_only": True}}
 
     def get_is_primary(self, obj):
         return True if obj.inventories.filter(is_primary=True).exists() else False
@@ -603,7 +631,9 @@ class ProductSerializer(FavoriteMixin, serializers.ModelSerializer):
         return data
 
 
-class ProductListSerializer(ProductSerializer):
+class ProductListSerializer(serializers.ModelSerializer):
+    product_images = ProductMediaSerializer(many=True, source="images")
+
     class Meta:
         model = Product
         fields = (
@@ -795,9 +825,7 @@ class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
 
     def get_is_closed(self, obj):
         current_time = now()
-        current_op_hour = obj.opening_hours.filter(
-            weekday=current_time.weekday() + 1
-        ).first()
+        current_op_hour = obj.current_opening_hours
         if current_op_hour:
             return (
                 not current_op_hour.from_hour
@@ -869,9 +897,7 @@ class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
         return PhoneContactSerializer(phone_contacts, many=True).data
 
     def get_current_day_opening_hours(self, obj):
-        current_opening_hours = obj.opening_hours.filter(
-            weekday=now().weekday() + 1
-        ).first()
+        current_opening_hours = obj.current_opening_hours
         op_hour = {}
         if current_opening_hours:
             op_hour["from_hour"] = current_opening_hours.from_hour.strftime("%I:%M%p")
@@ -938,18 +964,25 @@ class GenericSerializer(serializers.Serializer):
         return serializer_class(context=context).to_representation(value)
 
     def get_serializer_for_instance(self, instance):
-        serializer_class = instance.__class__.__name__ + "Serializer"
-        if hasattr(sys.modules[__name__], serializer_class):
-            return getattr(sys.modules[__name__], serializer_class)
-        else:
-            raise NameError(_(f"name '{serializer_class}' is not defined"))
+        serializer_class = instance.__class__.__name__
+        return import_string(
+            store_settings.FAVORITES_SERIALIZERS_PATHS[serializer_class]
+        )
 
 
 class FavoriteExtraSerializer(serializers.ModelSerializer):
+    def get_favorite_extras_models():
+        extras_list = []
+
+        for key, value in settings.FAVORITE_TYPES.items():
+            if "extras" in value:
+                extras_list.extend(value["extras"].keys())
+        return extras_list
+
     content_object = GenericSerializer(read_only=True)
     object_id = serializers.IntegerField(min_value=1, write_only=True)
     object_type = serializers.ChoiceField(
-        write_only=True, choices=store_settings.EXTRA_FAVORITE_TYPES
+        write_only=True, choices=get_favorite_extras_models()
     )
 
     class Meta:
@@ -967,7 +1000,7 @@ class FavoriteSerializer(serializers.ModelSerializer):
     extras = FavoriteExtraSerializer(many=True)
     object_id = serializers.IntegerField(min_value=1, write_only=True)
     object_type = serializers.ChoiceField(
-        write_only=True, choices=store_settings.FAVORITE_TYPES
+        write_only=True, choices=list(store_settings.FAVORITE_TYPES.keys())
     )
 
     class Meta:
@@ -978,75 +1011,148 @@ class FavoriteSerializer(serializers.ModelSerializer):
             "extras",
             "object_id",
             "object_type",
+            "name",
         )
+        extra_kwargs = {
+            "name": {"required": True},
+        }
 
     def _lookup_validation(self, data):
-        content_type = ContentType.objects.get_for_model(type(data["content_object"]))
-        extras_lookup_content_type = {}
-        for extra in data["extras"]:
-            content_type_extra = extra.model
-            extras_lookup_content_type[content_type_extra] = extra
+        content_type = ContentType.objects.get_for_model(data["content_object"])
         queryset = Favorite.objects.filter(
             content_type=content_type,
             object_id=data["content_object"].id,
             user=self.context["request"].user,
-        )
-        for key in extras_lookup_content_type.keys():
-            queryset = queryset.filter(
-                extras__content_type=ContentType.objects.get_for_model(key),
-                extras__object_id__in=[
-                    extra.id for extra in extras_lookup_content_type[key]
-                ],
-            )
-        if queryset.exists():
+        ).prefetch_related("extras")
+
+        if not queryset.exists():
+            return
+
+        for favorite in queryset:
+            content_objects = [
+                instance.content_object for instance in favorite.extras.all()
+            ]
+            if data["extras"] == content_objects:
+                raise serializers.ValidationError(
+                    _(f"You cannot favorite the same item twice")
+                )
+
+    def get_object(self, model: models.Model, id: int):
+        try:
+            object = model.objects.get(pk=id)
+            return object
+        except model.DoesNotExist:
             raise serializers.ValidationError(
-                _(f"You cannot favorite the same item twice")
+                _(f"{model.__name__} with the id of {id} does not exist")
             )
 
     def validate(self, attrs):
         validated_data = super().validate(attrs)
+        extras_data = {}
+        name = validated_data["name"]
         object_type = validated_data["object_type"]
-        if object_type not in store_settings.FAVORITE_TYPES and hasattr(
-            sys.modules[__name__], object_type
-        ):
-            raise serializers.ValidationError(
-                _(f"Cannot resolve keyword '{object_type}' into an object type")
-            )
-        object_type_model = getattr(sys.modules[__name__], object_type)
+        object_type_model = import_string(
+            store_settings.FAVORITE_TYPES[object_type]["path"]
+        )
         object_instance = get_object_or_404(
             object_type_model, pk=validated_data["object_id"]
         )
-        extras_data = {}
-        extras = []
         for extra in validated_data["extras"]:
-            extra_object_type = extra["object_type"]
-            if extra_object_type not in store_settings.EXTRA_FAVORITE_TYPES:
+            if (
+                extra["object_type"]
+                not in store_settings.FAVORITE_TYPES[object_type]["extras"]
+            ):
                 raise serializers.ValidationError(
-                    _(
-                        f"Cannot resolve keyword '{extra_object_type}' into an object type"
-                    )
+                    _(f"{extra['object_type']} Cannot be extra of {object_type}")
                 )
-            extras_data.setdefault(extra_object_type, [])
-            extras_data[extra_object_type].append(extra["object_id"])
-        try:
-            for key in extras_data.keys():
-                model_class = getattr(sys.modules[__name__], key, None)
-                if model_class:
-                    extras.append(model_class.objects.filter(id__in=extras_data[key]))
-        except ObjectDoesNotExist:
-            raise serializers.ValidationError(_("No matches the given query."))
+            extra_object_type = extra["object_type"]
+            extras_data.setdefault(
+                extra_object_type,
+                {
+                    "model": import_string(
+                        store_settings.FAVORITE_TYPES[object_type]["extras"][
+                            extra_object_type
+                        ]["path"]
+                    ),
+                    "ids": [],
+                },
+            )
+            extras_data[extra_object_type]["ids"].append(extra["object_id"])
 
+        extras = []
+        for key, value in extras_data.items():
+            type = store_settings.FAVORITE_TYPES[object_type]["extras"][key]["type"]
+            if len(value["ids"]) > 1 and type == store_settings.SIGNLE_FAVORITE_EXTRA:
+                raise serializers.ValidationError(_(f"Cannot set multiple {key}s"))
+            for id in value["ids"]:
+                extras.append(
+                    self.get_object(value["model"], id)
+                )  # get objects of extras instead of ids
+        extras = list(set(extras))  # remove duplicated extras
         validated_data = {
             "content_object": object_instance,
             "extras": extras,
+            "name": name,
         }
         self._lookup_validation(validated_data)
         return validated_data
 
     def create(self, validated_data):
-        favorite = Favorite.add_favorite(
-            validated_data["content_object"],
-            self.context["request"].user,
-            validated_data["extras"],
-        )
+        try:
+            favorite = Favorite.add_favorite(
+                content_object=validated_data["content_object"],
+                user=self.context["request"].user,
+                name=validated_data["name"],
+                extras=validated_data["extras"],
+            )
+        except ValidationError as e:
+            raise serializers.ValidationError(detail=e.message_dict)
         return favorite
+
+
+class WalletMediaSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = WalletMedia
+        fields = "__all__"
+
+
+class WalletSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = Wallet
+        fields = ["id", "user", "balance", "name", "media_image", "image_url"]
+        extra_kwargs = {
+            "user": {"read_only": True},
+        }
+
+
+class WalletTopUpSerializer(serializers.Serializer):
+    amount = serializers.DecimalField(
+        max_digits=10, decimal_places=2, min_value=1, required=True
+    )
+    payment_method = serializers.PrimaryKeyRelatedField(
+        queryset=PaymentMethod.objects.filter(
+            payment_provider__in=[
+                store_settings.TAP_CREDIT_CARD,
+                store_settings.TAP_KNET,
+                store_settings.TAP_ALL,
+            ]
+        ),
+        required=True,
+    )
+
+    def top_up_wallet(self, wallet):
+        amount = self.validated_data["amount"]
+        payment_method = self.validated_data["payment_method"]
+        return wallet.top_up_wallet(amount=amount, payment_method=payment_method)
+
+
+class WalletTransactionSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = WalletTransaction
+        fields = [
+            "id",
+            "type",
+            "amount",
+            "created_at",
+            "wallet",
+        ]
