@@ -26,6 +26,7 @@ from ob_dj_store.apis.stores.filters import (
     FavoriteFilter,
     InventoryFilter,
     OrderFilter,
+    PaymentMethodFilter,
     ProductFilter,
     StoreFilter,
     VariantFilter,
@@ -47,7 +48,12 @@ from ob_dj_store.apis.stores.rest.serializers.serializers import (
     ShippingMethodSerializer,
     StoreSerializer,
     TaxSerializer,
+    WalletMediaSerializer,
+    WalletSerializer,
+    WalletTopUpSerializer,
+    WalletTransactionSerializer,
 )
+from ob_dj_store.core.stores.gateway.tap.utils import TapException
 from ob_dj_store.core.stores.models import (
     Cart,
     CartItem,
@@ -62,6 +68,8 @@ from ob_dj_store.core.stores.models import (
     ShippingMethod,
     Store,
     Tax,
+    Wallet,
+    WalletMedia,
 )
 from ob_dj_store.core.stores.models._inventory import Inventory
 
@@ -93,7 +101,7 @@ class StoreView(
 
     def get_queryset(self):
         queryset = super().get_queryset()
-        queryset.select_related("opening_hours")
+        queryset = queryset.prefetch_related("opening_hours")
         if self.action == "favorites":
             favorite_store_ids = Favorite.objects.favorites_for_model(
                 Store, self.request.user
@@ -188,29 +196,6 @@ class StoreView(
         serializer = self.get_serializer(page, many=True)
         return self.get_paginated_response(serializer.data)
 
-    @swagger_auto_schema(
-        operation_summary="Add or Remove Store from Favorites",
-        operation_description="""
-            Add or Remove Store from Favorites
-        """,
-        tags=[
-            "Store",
-        ],
-    )
-    @action(
-        detail=True,
-        methods=["GET"],
-        url_path="favorite",
-    )
-    def favorite(self, request, *args, **kwargs):
-        instance = self.get_object()
-        try:
-            Favorite.objects.favorite_for_user(instance, request.user).delete()
-        except Favorite.DoesNotExist:
-            Favorite.add_favorite(instance, request.user)
-        serializer = StoreSerializer(instance=instance, context={"request": request})
-        return Response(serializer.data)
-
     @swagger_auto_schema(
         operation_summary="Retrieve count of store's products",
         operation_description="""
@@ -231,10 +216,13 @@ class StoreView(
             product_variants__inventories__store=store
         ).values_list("id", flat=True)
         content_type = ContentType.objects.get_for_model(Product)
-        favorites_count = Favorite.objects.filter(
-            content_type=content_type,
-            object_id__in=products_ids,
-        ).count()
+        favorites_count = (
+            Favorite.objects.filter(
+                content_type=content_type, object_id__in=products_ids, user=request.user
+            ).count()
+            if request.user.id
+            else 0
+        )
         menu = Product.objects.filter(
             product_variants__inventories__store=store,
             category__isnull=False,
@@ -295,7 +283,6 @@ class CartView(
         permissions.IsAuthenticated,
     ]
     queryset = Cart.objects.all()
-    lookup_value_regex = "[0-9]+"
 
     def get_object(self):
         return self.request.user.cart
@@ -564,14 +551,7 @@ class ProductView(
             ).values_list("object_id", flat=True)
 
             queryset = self.queryset.filter(pk__in=favorite_product_ids)
-        return queryset
-
-    def get_object(self):
-
-        instance = self.get_queryset().filter(pk=self.kwargs["pk"]).distinct().first()
-        if not instance:
-            raise Http404("No Product matches the given query.")
-        return instance
+        return queryset.distinct()
 
     @swagger_auto_schema(
         operation_summary="Retrieve A Product",
@@ -672,6 +652,9 @@ class CategoryViewSet(
     filterset_class = CategoryFilter
     lookup_value_regex = "[0-9]+"
 
+    def get_queryset(self):
+        return super().get_queryset().prefetch_related("products__images")
+
     def get_object(self):
         store_id = self.request.query_params.get("store", None)
         instance_pk = self.kwargs["pk"]
@@ -697,6 +680,7 @@ class CategoryViewSet(
                     "products",
                     queryset=product_queryset.distinct(),
                 ),
+                "subcategories__products__images",
             ).get(pk=instance_pk)
         except Category.DoesNotExist:
             raise Http404("No Category matches the given query.")
@@ -798,14 +782,9 @@ class PaymentMethodViewSet(
     permission_classes = [
         permissions.IsAuthenticated,
     ]
-    queryset = PaymentMethod.objects.all()
-
-    def get_queryset(self):
-        try:
-            store = Store.objects.get(pk=self.kwargs["store_pk"])
-        except ObjectDoesNotExist:
-            raise ValidationError(_(f"Store does not Exist"))
-        return store.payment_methods.filter(is_active=True)
+    queryset = PaymentMethod.objects.filter(is_active=True)
+    filterset_class = PaymentMethodFilter
+    lookup_value_regex = "[0-9]+"
 
     @swagger_auto_schema(
         operation_summary="List Payment Methods",
@@ -947,3 +926,147 @@ class FavoriteViewSet(
     )
     def destroy(self, request, *args, **kwargs):
         return super().destroy(request, *args, **kwargs)
+
+
+class WalletViewSet(
+    mixins.ListModelMixin,
+    mixins.RetrieveModelMixin,
+    mixins.UpdateModelMixin,
+    viewsets.GenericViewSet,
+):
+    queryset = Wallet.objects.all()
+    serializer_class = WalletSerializer
+    permission_classes = [
+        permissions.IsAuthenticated,
+    ]
+
+    def get_queryset(self):
+        return Wallet.objects.filter(user=self.request.user).select_related(
+            "media_image"
+        )
+
+    @swagger_auto_schema(
+        operation_summary="Get User Wallet",
+        operation_description="""
+                Get User Wallet
+            """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    def retrieve(
+        self, request: Request, *args: typing.Any, **kwargs: typing.Any
+    ) -> Response:
+        return super().retrieve(request=request, *args, **kwargs)
+
+    @swagger_auto_schema(
+        operation_summary="Update user wallet",
+        operation_description="""
+            Update a wallet
+        """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    def partial_update(self, request: Request, *args: typing.Any, **kwargs: typing.Any):
+        return super().partial_update(request, *args, **kwargs)
+
+    @swagger_auto_schema(
+        operation_summary="List User Wallets",
+        operation_description="""
+                List User Wallets
+            """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    def list(
+        self, request: Request, *args: typing.Any, **kwargs: typing.Any
+    ) -> Response:
+        return super().list(request=request, *args, **kwargs)
+
+    @swagger_auto_schema(
+        operation_summary="top up a wallet",
+        operation_description="""
+                top up a user wallet with tap payment
+            """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    @action(
+        methods=["POST"],
+        detail=True,
+        url_path="top-up",
+        url_name="top-up",
+        serializer_class=WalletTopUpSerializer,
+    )
+    def top_up_wallet(
+        self, request: Request, *args: typing.Any, **kwargs: typing.Any
+    ) -> Response:
+        serializer = self.get_serializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+        instance = self.get_object()
+        try:
+            payment_url = serializer.top_up_wallet(instance)
+        except TapException as err:
+            raise ValidationError({"tap": _(f"{str(err)}")})
+        return Response({"payment_url": payment_url}, status=status.HTTP_200_OK)
+
+    @swagger_auto_schema(
+        operation_summary="List Wallet's transactions",
+        operation_description="""
+                list user's wallet transactions(debit,credit)
+            """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    @action(
+        methods=[
+            "GET",
+        ],
+        detail=True,
+        url_path="transactions",
+        serializer_class=WalletTransactionSerializer,
+    )
+    def transactions(
+        self, request: Request, *args: typing.Any, **kwargs: typing.Any
+    ) -> Response:
+        queryset = self.get_object().transactions.all()
+        page = self.paginate_queryset(queryset)
+        if page is not None:
+            serializer = self.get_serializer(page, many=True)
+            return self.get_paginated_response(serializer.data)
+
+        serializer = self.get_serializer(queryset, many=True)
+        return Response(serializer.data)
+
+    @swagger_auto_schema(
+        operation_summary="List Wallet selection images",
+        operation_description="""
+                list wallet selection images
+            """,
+        tags=[
+            "Wallet",
+        ],
+    )
+    @action(
+        methods=[
+            "GET",
+        ],
+        detail=False,
+        url_path="images",
+        serializer_class=WalletMediaSerializer,
+    )
+    def images(
+        self, request: Request, *args: typing.Any, **kwargs: typing.Any
+    ) -> Response:
+        queryset = WalletMedia.objects.filter(is_active=True)
+        page = self.paginate_queryset(queryset)
+        if page is not None:
+            serializer = self.get_serializer(page, many=True)
+            return self.get_paginated_response(serializer.data)
+
+        serializer = self.get_serializer(queryset, many=True)
+        return Response(serializer.data)
