@@ -1,5 +1,6 @@
 from rest_framework import serializers
 
+from ob_dj_store.apis.stores.rest.serializers.serializers import OrderSerializer
 from ob_dj_store.core.stores.gateway.tap.models import TapPayment
 
 
@@ -28,12 +29,10 @@ class TapPaymentSerializer(serializers.ModelSerializer):
             "callback_response",
             "charge_id",
         ]
-        extra_kwargs = {"payment": {"write_only": True}}
 
-    def create(self, validated_data):
-        payment = validated_data.get("payment")
-        source = validated_data.get("source")
-        user = self.context.get("request").user
-        instance = TapPayment.objects.create(payment=payment, source=source, user=user)
-
-        return instance
+    def to_representation(self, instance):
+        representation = super().to_representation(instance)
+        representation["orders"] = OrderSerializer(
+            instance.payment.orders.all(), many=True
+        ).data
+        return representation
