@@ -99,18 +99,11 @@ class CategoryAdmin(admin.ModelAdmin):
     list_display = ["name", "is_active", "parent", "image"]
     search_fields = [
         "name",
-        "parent__name",
     ]
     list_filter = [
         "is_active",
     ]
 
-    def save_model(self, request, obj, form, change) -> None:
-        from ob_dj_store.core.stores.receivers import create_media_thumbnails
-
-        create_media_thumbnails(None, obj, None)
-        return super().save_model(request, obj, form, change)
-
 
 class ProductVariantAdmin(admin.ModelAdmin):
     inlines = [
@@ -123,6 +116,9 @@ class ProductVariantAdmin(admin.ModelAdmin):
     ]
     search_fields = ["name", "product__name", "sku"]
 
+    def get_queryset(self, request):
+        return super().get_queryset(request).prefetch_related("inventories")
+
 
 class ProductAdmin(admin.ModelAdmin):
     list_display = ["id", "name", "category", "type", "is_active"]
@@ -227,7 +223,7 @@ class PaymentAdmin(admin.ModelAdmin):
         "method__payment_provider",
         "status",
     ]
-    search_fields = ["orders", "user__email"]
+    search_fields = ["orders__store__name", "user__email"]
 
 
 class InventoryAdmin(admin.ModelAdmin):
@@ -250,6 +246,9 @@ class InventoryAdmin(admin.ModelAdmin):
         "is_uncountable",
     ]
 
+    def get_queryset(self, request):
+        return super().get_queryset(request).prefetch_related("store")
+
 
 class TaxAdmin(admin.ModelAdmin):
     list_display = [
@@ -278,7 +277,7 @@ class WalletTransactionAdmin(admin.ModelAdmin):
         "type",
     ]
     search_fields = [
-        "wallet__user_email",
+        "wallet__user__email",
     ]
 
 
@@ -289,6 +288,10 @@ class WalletAdmin(admin.ModelAdmin):
     ]
 
 
+class WalletMediaAdmin(admin.ModelAdmin):
+    list_display = ("id", "image", "image_thumbnail_medium", "is_active")
+
+
 admin.site.register(models.Store, StoreAdmin)
 admin.site.register(models.ShippingMethod, ShippingMethodAdmin)
 admin.site.register(models.PaymentMethod, PaymentMethodAdmin)
@@ -306,3 +309,4 @@ admin.site.register(models.Inventory, InventoryAdmin)
 admin.site.register(models.Tax, TaxAdmin)
 admin.site.register(models.WalletTransaction, WalletTransactionAdmin)
 admin.site.register(models.Wallet, WalletAdmin)
+admin.site.register(models.WalletMedia, WalletMediaAdmin)
