@@ -93,7 +93,7 @@ class TapPayment(models.Model):
 
     @property
     def amount(self):
-        return self.payment.amount
+        return self.payment.total_payment
 
     def callback_update(self, tap_payload):
         if self.status == self.Status.INITIATED:
