@@ -8,7 +8,11 @@ from ob_dj_store.core.stores.managers import (
     ProductManager,
     ProductVariantManager,
 )
-from ob_dj_store.utils.helpers import category_media_upload_to, product_media_upload_to
+from ob_dj_store.utils.helpers import (
+    category_media_upload_to,
+    product_media_upload_to,
+    product_variant_media_upload_to,
+)
 from ob_dj_store.utils.model import DjangoModelCleanMixin
 
 
@@ -262,6 +266,15 @@ class ProductVariant(DjangoModelCleanMixin, models.Model):
     sku = models.CharField(max_length=100, null=True, blank=True)
     plu = models.CharField(max_length=40, unique=True, null=True, blank=True)
     external_id = models.CharField(max_length=40, unique=True, null=True, blank=True)
+    image = models.ImageField(
+        upload_to=product_variant_media_upload_to,
+        null=True,
+        blank=True,
+        help_text="optional image field",
+    )
+    image_thumbnail_medium = models.ImageField(
+        upload_to="product_variant_media/", null=True, blank=True
+    )
     # Audit fields
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
@@ -319,8 +332,6 @@ class ProductMedia(DjangoModelCleanMixin, models.Model):
         verbose_name = _("product media")
         verbose_name_plural = _("Product medias")
 
-    def save(self, **kwargs):
-        from ob_dj_store.core.stores.receivers import create_media_thumbnails
-
-        create_media_thumbnails(None, self, None)
-        return super().save(**kwargs)
+    @property
+    def name(self):
+        return f"{self.product.name}_{self.order_value}"
