@@ -1,6 +1,8 @@
 import math
 
 import pycountry
+from django.core.exceptions import ValidationError
+from django.utils.translation import gettext_lazy as _
 
 
 def get_data_dict(instance):
@@ -53,13 +55,19 @@ def distance(origin, destination):
 
 def get_currency_by_country(country):
     country = pycountry.countries.get(name=country)
-    if not country:
-        return None
-    return pycountry.currencies.get(numeric=country.numeric).alpha_3
+    currency = pycountry.currencies.get(numeric=country.numeric)
+    return currency.alpha_3
 
 
 def get_country_by_currency(currency):
     currency = pycountry.currencies.get(alpha_3=currency)
-    if not currency:
-        return None
-    return pycountry.countries.get(numeric=currency.numeric).alpha_2
+    country = pycountry.countries.get(numeric=currency.numeric)
+    return country.alpha_2
+
+
+def validate_currency(value):
+    if not pycountry.currencies.get(alpha_3=value):
+        raise ValidationError(
+            _("%(value)s is not a currency"),
+            params={"value": value},
+        )
